# -*- coding: utf-8 -*-
"""
-------------------------------
(c) Universite catholique de Louvain, 2019

Creation : 2019 by O. Lantsoght
Last update : 2019

version MBsysC v1.11.2

-------------------------------
Portable Python interface to MBsysC using Ctypes.

Definition of CFUNCTYPE needed for callback functions.

"""

import ctypes

# importing MBsysC related classes
from .forward_decl import MbsData_c
from .forward_decl import MbsDirdyn_c
from .forward_decl import MbsEquil_c
from .forward_decl import UserIoInfo_c
from .forward_decl import UserIO_c
from .forward_decl import UserModel_c
from .forward_decl import MbsInfos_c
from .forward_decl import MbsSensor_c


#==============================================================================
# Global parameter of the current module
#==============================================================================
__DEBUG__ = False


#==============================================================================
# Declaration of function to be linked with the libraries
#==============================================================================
# User
user_JointForces_wrap = ctypes.CFUNCTYPE(None, ctypes.POINTER(MbsData_c), ctypes.c_double)
user_load_post_wrap = ctypes.CFUNCTYPE(None, ctypes.POINTER(MbsData_c))
user_dirdyn_init_wrap = ctypes.CFUNCTYPE(ctypes.c_void_p, ctypes.POINTER(MbsData_c), ctypes.POINTER(MbsDirdyn_c))
user_dirdyn_loop_wrap = ctypes.CFUNCTYPE(ctypes.c_void_p, ctypes.POINTER(MbsData_c), ctypes.POINTER(MbsDirdyn_c))
user_dirdyn_finish_wrap = ctypes.CFUNCTYPE(ctypes.c_void_p, ctypes.POINTER(MbsData_c), ctypes.POINTER(MbsDirdyn_c))
user_equil_init_wrap = ctypes.CFUNCTYPE(None, ctypes.POINTER(MbsData_c), ctypes.POINTER(MbsEquil_c))
user_equil_loop_wrap = ctypes.CFUNCTYPE(None, ctypes.POINTER(MbsData_c), ctypes.POINTER(MbsEquil_c))
user_equil_finish_wrap = ctypes.CFUNCTYPE(None, ctypes.POINTER(MbsData_c), ctypes.POINTER(MbsEquil_c))
user_equil_fxe_wrap = ctypes.CFUNCTYPE(None, ctypes.POINTER(MbsData_c), ctypes.POINTER(ctypes.c_double))
user_Derivative_wrap = ctypes.CFUNCTYPE(None, ctypes.POINTER(MbsData_c))
user_DrivenJoints_wrap = ctypes.CFUNCTYPE(None, ctypes.POINTER(MbsData_c), ctypes.c_double)

user_cons_hJ_wrap = ctypes.CFUNCTYPE(None, ctypes.POINTER(ctypes.c_double), ctypes.POINTER(ctypes.POINTER(ctypes.c_double)), ctypes.POINTER(MbsData_c), ctypes.c_double)
user_cons_jdqd_wrap = ctypes.CFUNCTYPE(None, ctypes.POINTER(ctypes.c_double), ctypes.POINTER(MbsData_c), ctypes.c_double)
user_LinkForces_wrap = ctypes.CFUNCTYPE(ctypes.c_double, ctypes.c_double, ctypes.c_double, ctypes.POINTER(MbsData_c), ctypes.c_double, ctypes.c_int)
# Correctly working with static array: https://stackoverflow.com/questions/20752834/ctypes-initialize-array-of-arrays-and-pass-to-c-function
user_Link3DForces_wrap = ctypes.CFUNCTYPE(None, ctypes.POINTER(ctypes.c_double), ctypes.POINTER(ctypes.c_double), ctypes.POINTER(ctypes.c_double), ctypes.POINTER(ctypes.c_double), ctypes.POINTER(ctypes.c_double), ctypes.POINTER(ctypes.c_double), ctypes.POINTER(MbsData_c), ctypes.c_double, ctypes.c_int)
user_ExtForces_wrap = ctypes.CFUNCTYPE(None, ctypes.POINTER(ctypes.c_double), ctypes.POINTER(ctypes.c_double), ctypes.POINTER(ctypes.c_double), ctypes.POINTER(ctypes.c_double), ctypes.POINTER(ctypes.c_double), ctypes.POINTER(ctypes.c_double), ctypes.POINTER(MbsData_c), ctypes.c_double, ctypes.c_int)

mbs_new_user_IO_wrap = ctypes.CFUNCTYPE(ctypes.POINTER(UserIO_c), ctypes.POINTER(UserIoInfo_c))
mbs_new_user_model_wrap = ctypes.CFUNCTYPE(ctypes.POINTER(UserModel_c))
mbs_delete_user_model_wrap = ctypes.CFUNCTYPE(None, ctypes.POINTER(UserModel_c))
mbs_delete_user_IO_wrap = ctypes.CFUNCTYPE(None, ctypes.POINTER(UserIO_c))
mbs_get_user_IO_size_wrap = ctypes.CFUNCTYPE(None, ctypes.POINTER(ctypes.c_int), ctypes.POINTER(ctypes.c_int), ctypes.POINTER(ctypes.c_int))
mbs_load_user_model_xml_wrap= ctypes.CFUNCTYPE(None, ctypes.POINTER(MbsInfos_c), ctypes.POINTER(UserModel_c))
mbs_bind_user_model_wrap = ctypes.CFUNCTYPE(None, ctypes.POINTER(MbsInfos_c), ctypes.POINTER(UserModel_c))
mbs_print_user_model_wrap = ctypes.CFUNCTYPE(None, ctypes.POINTER(UserModel_c))
mbs_get_user_model_size_wrap = ctypes.CFUNCTYPE(None, ctypes.POINTER(ctypes.c_int))
mbs_get_user_model_list_wrap = ctypes.CFUNCTYPE(None, ctypes.POINTER(ctypes.c_int)) 

# Symbolic
mbs_link_wrap = ctypes.CFUNCTYPE(None, ctypes.POINTER(ctypes.POINTER(ctypes.c_double)), ctypes.POINTER(ctypes.POINTER(ctypes.c_double)), ctypes.POINTER(ctypes.c_double), ctypes.POINTER(ctypes.c_double), ctypes.POINTER(ctypes.c_double), ctypes.POINTER(MbsData_c), ctypes.c_double)
mbs_link3D_wrap = ctypes.CFUNCTYPE(None, ctypes.POINTER(ctypes.POINTER(ctypes.c_double)), ctypes.POINTER(ctypes.POINTER(ctypes.c_double)), ctypes.POINTER(MbsData_c), ctypes.c_double)
mbs_extforces_wrap = ctypes.CFUNCTYPE(None, ctypes.POINTER(ctypes.POINTER(ctypes.c_double)), ctypes.POINTER(ctypes.POINTER(ctypes.c_double)), ctypes.POINTER(MbsData_c), ctypes.c_double)
mbs_accelred_wrap = ctypes.CFUNCTYPE(ctypes.c_int, ctypes.POINTER(MbsData_c), ctypes.c_double)
mbs_dirdyna_wrap = ctypes.CFUNCTYPE(None, ctypes.POINTER(ctypes.POINTER(ctypes.c_double)), ctypes.POINTER(ctypes.c_double), ctypes.POINTER(MbsData_c), ctypes.c_double)
mbs_invdyna_wrap = ctypes.CFUNCTYPE(None, ctypes.POINTER(ctypes.c_double), ctypes.POINTER(MbsData_c), ctypes.c_double)
mbs_cons_hJ_wrap = ctypes.CFUNCTYPE(None, ctypes.POINTER(ctypes.c_double), ctypes.POINTER(ctypes.POINTER(ctypes.c_double)), ctypes.POINTER(MbsData_c), ctypes.c_double)
mbs_cons_jdqd_wrap = ctypes.CFUNCTYPE(None, ctypes.POINTER(ctypes.c_double), ctypes.POINTER(MbsData_c), ctypes.c_double)
mbs_sensor_wrap = ctypes.CFUNCTYPE(None, ctypes.POINTER(MbsSensor_c), ctypes.POINTER(MbsData_c), ctypes.c_int)
mbs_gensensor_wrap = ctypes.CFUNCTYPE(None, ctypes.POINTER(MbsSensor_c), ctypes.POINTER(MbsData_c), ctypes.c_int)

# Message functions
mbs_messages_wrap = ctypes.CFUNCTYPE(None, ctypes.c_char_p)
