# -*- coding: utf-8 -*-
"""
-------------------------------
(c) Universite catholique de Louvain, 2019

Creation : 2019 by O. Lantsoght
Last update : 2019

version MBsysC v1.11.2

-------------------------------
Portable Python interface to MBsysC using Ctypes.
While generating the libraries:
  - SENSORKIN MUST BE UNDEFINED
  - REALTIME  MUST BE UNDEFINED

  - PRJ_FCT_PTR MUST BE DEFINED

Importing MBsysC libraries

"""
import os
import numpy
import ctypes
import platform

#==============================================================================
# Global parameter of the current module
#==============================================================================
__MODULE_DIR__ = os.path.dirname(os.path.abspath(__file__))
__OS__ = platform.uname()[0]

#==============================================================================
# Path to the libraries
#==============================================================================
try:
    from .libpath import __LIB_PATH__
    if  __OS__ == "Windows":
        __LIB_PATH__ = os.path.join(__LIB_PATH__,'Release')
except ImportError:
    __LIB_PATH__ = os.path.join(__MODULE_DIR__,'../libs64')
    if __OS__ == "Linux":
        __LIB_PATH__ = os.path.join(__LIB_PATH__,'linux')
    elif __OS__ == "Darwin":
        __LIB_PATH__ = os.path.join(__LIB_PATH__,'darwin')
    elif __OS__ == "Windows":
        __LIB_PATH__ = os.path.join(__LIB_PATH__,'windows')

#==============================================================================
# Loading libraries
#==============================================================================
if __OS__ == "Linux":
    ctypes.DEFAULT_MODE = ctypes.RTLD_GLOBAL
elif __OS__ == "Windows":
    # Also loading Windows specific libraries
    libxml2_2 = numpy.ctypeslib.load_library("libxml2-2",__LIB_PATH__)
    libopenblas   = numpy.ctypeslib.load_library("libopenblas",__LIB_PATH__)
elif not(__OS__ == "Darwin"):
    raise ImportError("Unsupported Operating System identifier: "+__OS__)


libutilities = numpy.ctypeslib.load_library("libMBsysC_utilities",__LIB_PATH__)
libloadXML   = numpy.ctypeslib.load_library("libMBsysC_loadXML",__LIB_PATH__)
libmodules   = numpy.ctypeslib.load_library("libMBsysC_module",__LIB_PATH__)
