# -*- coding: utf-8 -*-
"""
-------------------------------
(c) Universite catholique de Louvain, 2019

Creation : 2019 by O. Lantsoght
Last update : 2019

version MBsysC v1.11.2

-------------------------------
Portable Python interface to MBsysC using Ctypes.
While generating the libraries:
  - SENSORKIN    MUST BE UNDEFINED
  - REALTIME     MUST BE UNDEFINED
  - ACCELRED     MUST BE UNDEFINED
  - MBS_SIMULINK MUST BE UNDEFINED

  - PRJ_FCT_PTR MUST BE DEFINED
  - DIRDYNARED  MUST BE DEFINED
  - INVDYNARED  MUST BE DEFINED

Declaration of MBsysC structure related to mbs_aux.h file in MBsysC.

"""

import ctypes

# importing MBsysC related classes
from .forward_decl import MbsAux_c
from .forward_decl import MbsSensor_c


#==============================================================================
# Global parameter of the current module
#==============================================================================
__DEBUG__ = True


#==============================================================================
# MbsAux
#==============================================================================
MbsAux_c._fields_ = [
        ("norm_h",      ctypes.c_double),
        ("NRerr",       ctypes.c_double),
        ("MAX_NR_ITER", ctypes.c_int),

        ("h",           ctypes.POINTER(ctypes.c_double)),
        ("Jac",         ctypes.POINTER(ctypes.POINTER(ctypes.c_double))),
        ("huserc",      ctypes.POINTER(ctypes.c_double)),
        ("Juserc",      ctypes.POINTER(ctypes.POINTER(ctypes.c_double))),
        ("mJv",         ctypes.POINTER(ctypes.POINTER(ctypes.c_double))),
        ("ind_mJv",     ctypes.POINTER(ctypes.c_int)),
        ("Qc",          ctypes.POINTER(ctypes.c_double)),

        ("mJv_h",       ctypes.POINTER(ctypes.c_double)),

        ("nquc",        ctypes.c_int),
        ("iquc",        ctypes.POINTER(ctypes.c_int)),
        ("Juct",        ctypes.POINTER(ctypes.POINTER(ctypes.c_double))),

        ("Bvuc",        ctypes.POINTER(ctypes.POINTER(ctypes.c_double))),

        ("Jvtlambda",   ctypes.POINTER(ctypes.c_double)),
        ("Jvt",         ctypes.POINTER(ctypes.POINTER(ctypes.c_double))),
        ("ind_Jvt",     ctypes.POINTER(ctypes.c_int)),


        ("jdqd",        ctypes.POINTER(ctypes.c_double)),
        ("jdqduserc",   ctypes.POINTER(ctypes.c_double)),

        ("bp",          ctypes.POINTER(ctypes.c_double)),

        ("psens",       ctypes.POINTER(MbsSensor_c)),
        
        ("q_save",      ctypes.POINTER(ctypes.c_double)),
        ("close_anim",  ctypes.c_int),
        
        ("c",           ctypes.POINTER(ctypes.c_double)),
        ("M",           ctypes.POINTER(ctypes.POINTER(ctypes.c_double))),
        ("F",           ctypes.POINTER(ctypes.c_double)),

        ("BtMvu",       ctypes.POINTER(ctypes.POINTER(ctypes.c_double))),
        ("BtMvv",       ctypes.POINTER(ctypes.POINTER(ctypes.c_double))),
        ("BtMB",        ctypes.POINTER(ctypes.POINTER(ctypes.c_double))),
        ("BtFv",        ctypes.POINTER(ctypes.c_double)),
        ("MBMb",        ctypes.POINTER(ctypes.c_double)),

        ("Mruc",        ctypes.POINTER(ctypes.POINTER(ctypes.c_double))),
        ("Fruc",        ctypes.POINTER(ctypes.c_double)),

        ("Mr",          ctypes.POINTER(ctypes.POINTER(ctypes.c_double))),
        ("Fr",          ctypes.POINTER(ctypes.c_double)),

        ("p_Mr",        ctypes.POINTER(ctypes.c_double)),

        ("compute_Qc_vec", ctypes.POINTER(ctypes.c_int)),
        
        ("Qact",        ctypes.POINTER(ctypes.c_double)),
        ("A",           ctypes.POINTER(ctypes.POINTER(ctypes.c_double))),
        ("ind_A",       ctypes.POINTER(ctypes.c_int)),

        ("b",           ctypes.POINTER(ctypes.c_double)),

        ("w",           ctypes.POINTER(ctypes.c_double)), 
        ("v",           ctypes.POINTER(ctypes.POINTER(ctypes.c_double))),
        
        
        ("phi",         ctypes.POINTER(ctypes.c_double)),
        ("R",           ctypes.POINTER(ctypes.c_double)),
        ("Rruc",        ctypes.POINTER(ctypes.c_double)),
        ("Rred",        ctypes.POINTER(ctypes.c_double)),
        
        ("x",           ctypes.c_double),
        ("y",           ctypes.POINTER(ctypes.c_double)),
        ("dydx",        ctypes.POINTER(ctypes.c_double))]
