# -*- coding: utf-8 -*-
"""
-------------------------------
(c) Universite catholique de Louvain, 2019

Creation : 2019 by O. Lantsoght
Last update : 2019

version MBsysC v1.11.2

-------------------------------
Portable Python interface to MBsysC using Ctypes.

While generating the libraries:
  - SENSORKIN MUST BE UNDEFINED
  - REALTIME  MUST BE UNDEFINED

  - PRJ_FCT_PTR MUST BE DEFINED

Declaration of MBsysC structure related to mbs_dirdyn_struct.h file in MBsysC.

"""

import ctypes

# importing MBsysC related classes
from .forward_decl import MbsDirdynOptions_c
from .forward_decl import MbsDirdyn_c
from .forward_decl import MbsAux_c
from .forward_decl import MbsBuffer_c
from .forward_decl import MbsGrowingBuffer_c


#==============================================================================
# Global parameter of the current module
#==============================================================================
__DEBUG__ = False


#==============================================================================
# MbsDirdynOptions_c
#==============================================================================
MbsDirdynOptions_c._fields_ = [
        ("t0", ctypes.c_double),
        ("tf", ctypes.c_double),
        ("dt0", ctypes.c_double),
        ("save2file", ctypes.c_int),
        ("resfilename", ctypes.c_char_p),
        ("respath", ctypes.c_char_p),
        ("animpath", ctypes.c_char_p),
        ("save_anim", ctypes.c_int),
        ("save_visu", ctypes.c_int),
        ("framerate", ctypes.c_int),
        ("saveperiod", ctypes.c_int),
        ("max_save_user", ctypes.c_int),
        ("buffersize", ctypes.c_int),
        ("realtime", ctypes.c_int),
        ("accelred", ctypes.c_int),
        ("flag_compute_Qc", ctypes.c_int),
        ("compute_all_uxd", ctypes.c_int), 
        ("compute_Qc", ctypes.POINTER(ctypes.c_int)),
        ("integrator", ctypes.c_int),
        ("verbose", ctypes.c_int),
        ("flag_stop_stiff", ctypes.c_int),
        ("flag_precise_dynamics", ctypes.c_int),
        ("flag_waypoint", ctypes.c_int),
        ("flag_solout_wp", ctypes.c_int),
        ("delta_t_wp", ctypes.c_double),
        ("nmax", ctypes.c_int),
        ("toler", ctypes.c_double),
        ("rtoler", ctypes.c_double),
        ("atoler", ctypes.c_double),
        ("dt_max", ctypes.c_double),
        ("n_freeze", ctypes.c_int),
        ("show_failed_closure", ctypes.c_int),
        ]


#==============================================================================
# MbsDirdyn_c
#==============================================================================
MbsDirdyn_c._fields_ =[
        ("options", ctypes.POINTER(MbsDirdynOptions_c)),
        ("mbs_aux", ctypes.POINTER(MbsAux_c)),
        ("tsim", ctypes.c_double),
        ("dt", ctypes.c_double),
        ("nState", ctypes.c_int),
        ("y", ctypes.POINTER(ctypes.c_double)),
        ("yout", ctypes.POINTER(ctypes.c_double)),
        ("yd", ctypes.POINTER(ctypes.c_double)),
        ("buffers", ctypes.POINTER(ctypes.POINTER(MbsBuffer_c))),
        ("buffer_visu", ctypes.POINTER(ctypes.POINTER(MbsBuffer_c))),
        ("user_buffer", ctypes.POINTER(MbsGrowingBuffer_c)),
        ("bufferNb", ctypes.c_int),
        ("savedArrays", ctypes.POINTER(ctypes.POINTER(ctypes.c_double))),
        ("SavePeriodCounter", ctypes.c_int)
        # Pointer to integrator related functions are not exposed
        ]