#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
-------------------------------
(c) Universite catholique de Louvain, 2019

Creation : 2019 by LB
Last update : 2019

version MBsysC v1.11.2

-------------------------------
Portable Python interface to MBsysC using Ctypes.


Declaration of MBsysC structure related to mbs_equil_struct.h header file in MBsysC.

"""
import ctypes

from .forward_decl import MbsEquil_c
from .forward_decl import MbsEquilOptions_c
from .forward_decl import MbsBuffer_c
from .forward_decl import MbsGrowingBuffer_c
from .forward_decl import MbsAux_c
from .forward_decl import MbsLpk_c

#==============================================================================
# MbsEquilOptions_c
#==============================================================================

MbsEquilOptions_c._fields_ = [
        ("method", ctypes.c_int),
        ("equitol",  ctypes.c_double),    
        ("verbose", ctypes.c_int),
        ("save2file", ctypes.c_int),    
        ("compute_uxd", ctypes.c_int),    
        ("grad_lpk", ctypes.c_int),   
        ("lpk_itermax", ctypes.c_int),
        ("lpk_relincr", ctypes.c_double),
        ("lpk_absincr", ctypes.c_double),
        ("lpk_equitol", ctypes.c_double),
        ("lpk_lintol", ctypes.c_double),
        ("resfilename", ctypes.c_char_p),        
        ("respath", ctypes.c_char_p),    
        ("animpath", ctypes.c_char_p), 
        ("mode", ctypes.c_int),
        ("relax", ctypes.c_int),
        ("relaxcoeff", ctypes.c_double),    
        ("relaxIterMax", ctypes.c_double),
        ("soft", ctypes.c_int),
        ("senstol", ctypes.c_double),
        ("devjac", ctypes.c_double),
        ("itermax", ctypes.c_int),
        ("visualize", ctypes.c_int),    
        ("clearmbsglobal", ctypes.c_int),
        ("nquch", ctypes.c_int),
        ("quch", ctypes.POINTER(ctypes.c_int)),
        ("xch_ptr", ctypes.POINTER(ctypes.POINTER(ctypes.c_double))),
        ("nxe", ctypes.c_int),
        ("xe_ptr", ctypes.POINTER(ctypes.POINTER(ctypes.c_double))),
        ("xe_index", ctypes.POINTER(ctypes.c_int)),
        ("saveperiod", ctypes.c_int),
        ("save_anim", ctypes.c_int),
        ("framerate", ctypes.c_int),
        ("buffersize", ctypes.c_int),
        ("max_save_user", ctypes.c_int)
        #void (* fxe_ptr)(MbsData*, double*);    //! pointer pointing on the .c function calculating the added equilibrum functions
        ]

#==============================================================================
# MbsEquil_c
#==============================================================================


MbsEquil_c._fields_ = [
        ("options", ctypes.POINTER(MbsEquilOptions_c)),
        ("aux", ctypes.POINTER(MbsAux_c)), 
        ("qd_saved", ctypes.POINTER(ctypes.c_double)),
        ("qdd_saved", ctypes.POINTER(ctypes.c_double)),
        ("nx", ctypes.c_int),
        ("x", ctypes.POINTER(ctypes.c_double)),
        ("x_ptr", ctypes.POINTER(ctypes.POINTER(ctypes.c_double))),
        ("Nux_saved", ctypes.c_int),
        ("nxch", ctypes.c_int),
        ("xch", ctypes.POINTER(ctypes.c_int)),
        ("nxe", ctypes.c_int),
        ("xe", ctypes.POINTER(ctypes.c_int)),
        ("nxs", ctypes.c_int),
        ("xs", ctypes.POINTER(ctypes.c_int)),
        ("nxns", ctypes.c_int),
        ("xns", ctypes.POINTER(ctypes.c_int)),
        ("norm_pk", ctypes.c_double),
        ("grad", ctypes.POINTER(ctypes.POINTER(ctypes.c_double))),
        ("grad_Rr", ctypes.POINTER(ctypes.POINTER(ctypes.c_double))),
        ("grad_uxd", ctypes.POINTER(ctypes.POINTER(ctypes.c_double))),
        ("grad_fxe", ctypes.POINTER(ctypes.POINTER(ctypes.c_double))),
        ("lpk", ctypes.POINTER(MbsLpk_c)), 
        ("F", ctypes.POINTER(ctypes.c_double)),
        ("R", ctypes.c_double),
        ("iter", ctypes.c_int),
        ("savePeriodCounter", ctypes.c_int),
        ("bufferNb", ctypes.c_int),
        ("buffers", ctypes.POINTER(ctypes.POINTER(MbsBuffer_c))),
        ("user_buffer", ctypes.POINTER(MbsGrowingBuffer_c)),
        ("savedArrays", ctypes.POINTER(ctypes.POINTER(ctypes.c_double))),
        ("success", ctypes.c_int)]