# -*- coding: utf-8 -*-
"""
-------------------------------
(c) Universite catholique de Louvain, 2019

Creation : 2019 by O. Lantsoght
Last update : 2019

version MBsysC v1.11.2

-------------------------------
Portable Python interface to MBsysC using Ctypes.

Setting arguments and return type of functions loaded from MBsysC libraries.

"""

import ctypes

# importing libraries
from .loadlibs import libutilities
from .loadlibs import libloadXML
from .loadlibs import libmodules

# importing MBsysC related classes
from .forward_decl import MbsData_c
from .forward_decl import MbsSensor_c
from .forward_decl import MbsInfos_c
from .forward_decl import MbsLoader_c
from .forward_decl import MbsPart_c
from .forward_decl import MbsDirdyn_c
from .forward_decl import MbsEquil_c
from .forward_decl import MbsEquilOptions_c
from .forward_decl import MbsModal_c
from .forward_decl import MbsInvdyn_c
from .forward_decl import MbsSolvekin_c

# importing wrapping functions
from .callback import mbs_messages_wrap



#==============================================================================
# Global parameter of the current module
#==============================================================================
__DEBUG__ = False


#==============================================================================
# Setting functions (C) arguments and/or return type
#==============================================================================
# utilities
libutilities.mbs_print_data.argtypes =[ctypes.POINTER(MbsData_c)]
libutilities.mbs_print_data.restype  = None

libutilities.mbs_reset_data.argtypes =[ctypes.POINTER(MbsData_c)]
libutilities.mbs_reset_data.restype  = None

libutilities.mbs_set_nb_userc.argtypes =[ctypes.POINTER(MbsData_c), ctypes.c_int]
libutilities.mbs_set_nb_userc.restype  = None

libutilities.mbs_set_qa.argtypes =[ctypes.POINTER(MbsData_c), ctypes.c_int]
libutilities.mbs_set_qa.restype  = None

libutilities.mbs_set_qdriven.argtypes =[ctypes.POINTER(MbsData_c), ctypes.c_int]
libutilities.mbs_set_qdriven.restype  = None

libutilities.mbs_set_qlocked.argtypes =[ctypes.POINTER(MbsData_c), ctypes.c_int]
libutilities.mbs_set_qlocked.restype  = None

libutilities.mbs_set_qu.argtypes =[ctypes.POINTER(MbsData_c), ctypes.c_int]
libutilities.mbs_set_qu.restype  = None

libutilities.mbs_set_qv.argtypes =[ctypes.POINTER(MbsData_c), ctypes.c_int]
libutilities.mbs_set_qv.restype  = None

libutilities.mbs_unset_qa.argtypes =[ctypes.POINTER(MbsData_c), ctypes.c_int]
libutilities.mbs_unset_qa.restype  = None

libutilities.mbs_empty_qa.argtypes = [ctypes.POINTER(MbsData_c)]
libutilities.mbs_empty_qa.restype  = None

libutilities.set_output_ext.argtypes =[ctypes.c_double, ctypes.c_char_p]
libutilities.set_output_ext.restypes = None

libutilities.define_output_vector_ext.argtypes =[ctypes.c_char_p, ctypes.c_int]
libutilities.define_output_vector_ext.restypes = None

libutilities.set_output_value.argtypes =[ctypes.c_double, ctypes.c_int, ctypes.c_char_p]
libutilities.set_output_value.restypes = None

libutilities.mbs_msg_default.argtypes =[ctypes.c_char_p]
libutilities.mbs_msg_default.restypes = None

libutilities.mbs_warning_msg_default.argtypes =[ctypes.c_char_p]
libutilities.mbs_warning_msg_default.restypes = None

libutilities.mbs_error_msg_default.argtypes =[ctypes.c_char_p]
libutilities.mbs_error_msg_default.restypes = None

libutilities.mbs_set_msg_fct_ptr.argtypes = [mbs_messages_wrap]
libutilities.mbs_set_msg_fct_ptr.restypes = None

libutilities.mbs_set_warning_msg_fct_ptr.argtypes = [mbs_messages_wrap]
libutilities.mbs_set_warning_msg_fct_ptr.restypes = None

libutilities.mbs_set_error_msg_fct_ptr.argtypes = [mbs_messages_wrap]
libutilities.mbs_set_error_msg_fct_ptr.restypes = None



# LoadXML
libloadXML.mbs_info_reader.argtypes =[ctypes.c_char_p]
libloadXML.mbs_info_reader.restype  = ctypes.POINTER(MbsInfos_c)

libloadXML.mbs_new_loader.argtypes = None
libloadXML.mbs_new_loader.restype  = ctypes.POINTER(MbsLoader_c)

libloadXML.mbs_load_with_loader.argtypes =[ctypes.c_char_p, ctypes.c_char_p, 
                                           ctypes.POINTER(MbsLoader_c)]
libloadXML.mbs_load_with_loader.restype  = ctypes.POINTER(MbsData_c)

libloadXML.mbs_delete_data.argtypes = [ctypes.POINTER(MbsData_c)]
libloadXML.mbs_delete_data.restype  = None

libloadXML.mbs_delete_loader.argtypes = [ctypes.POINTER(MbsLoader_c)]
libloadXML.mbs_delete_loader.restype  = None

libloadXML.mbs_delete_infos.argtypes = [ctypes.POINTER(MbsInfos_c)]
libloadXML.mbs_delete_infos.restype  = None

# modules
libmodules.mbs_new_sensor.argstypes = [ctypes.POINTER(MbsData_c)]
libmodules.mbs_new_sensor.restype = ctypes.POINTER(MbsSensor_c)

libmodules.allocate_sensor.argstypes = [ctypes.POINTER(MbsSensor_c), ctypes.c_int]
libmodules.allocate_sensor.restype = None

libmodules.init_sensor.argstypes = [ctypes.POINTER(MbsSensor_c), ctypes.c_int]
libmodules.init_sensor.restype = None


libmodules.mbs_new_part.argtypes =[ctypes.POINTER(MbsData_c)]
libmodules.mbs_new_part.restype  = ctypes.POINTER(MbsPart_c)

libmodules.mbs_run_part.argtypes =[ctypes.POINTER(MbsPart_c), ctypes.POINTER(MbsData_c)]
libmodules.mbs_run_part.restype  = None

libmodules.mbs_delete_part.argtypes =[ctypes.POINTER(MbsPart_c)]
libmodules.mbs_delete_part.restype  = None


libmodules.mbs_new_dirdyn.argtypes =[ctypes.POINTER(MbsData_c)]
libmodules.mbs_new_dirdyn.restype  = ctypes.POINTER(MbsDirdyn_c)

libmodules.mbs_run_dirdyn.argtypes =[ctypes.POINTER(MbsDirdyn_c), ctypes.POINTER(MbsData_c)]
libmodules.mbs_run_dirdyn.restype  = None

libmodules.mbs_dirdyn_init.argtypes =[ctypes.POINTER(MbsDirdyn_c), ctypes.POINTER(MbsData_c)]
libmodules.mbs_dirdyn_init.restype  = None

libmodules.mbs_dirdyn_loop.argtypes =[ctypes.POINTER(MbsDirdyn_c), ctypes.POINTER(MbsData_c)]
libmodules.mbs_dirdyn_loop.restype  = None

libmodules.mbs_dirdyn_finish.argtypes =[ctypes.POINTER(MbsDirdyn_c), ctypes.POINTER(MbsData_c)]
libmodules.mbs_dirdyn_finish.restype  = None

libmodules.mbs_delete_dirdyn.argtypes =[ctypes.POINTER(MbsDirdyn_c), ctypes.POINTER(MbsData_c)]
libmodules.mbs_delete_dirdyn.restype  = None


libmodules.mbs_new_equil.argtypes =[ctypes.POINTER(MbsData_c)]
libmodules.mbs_new_equil.restype  = ctypes.POINTER(MbsEquil_c)

libmodules.mbs_run_equil.argtypes =[ctypes.POINTER(MbsEquil_c), ctypes.POINTER(MbsData_c)]
libmodules.mbs_run_equil.restype  = None

libmodules.mbs_delete_equil.argtypes =[ctypes.POINTER(MbsEquil_c), ctypes.POINTER(MbsData_c)]
libmodules.mbs_delete_equil.restype  = None

libmodules.mbs_equil_exchange.argtypes = [ctypes.POINTER(MbsEquilOptions_c)]
libmodules.mbs_equil_exchange.restypes = None

libmodules.mbs_equil_addition.argtypes = [ctypes.POINTER(MbsEquilOptions_c)]
libmodules.mbs_equil_addition.restypes = None

libmodules.mbs_print_equil.argtypes = [ctypes.POINTER(MbsEquil_c)]
libmodules.mbs_print_equil.restypes = None

libmodules.mbs_equil_set_variable.argtypes = [ctypes.POINTER(MbsEquilOptions_c), 
                                              ctypes.c_void_p, ctypes.c_int]
libmodules.mbs_equil_set_variable.restypes = None

libmodules.mbs_equil_add_variable.argtypes = [ctypes.POINTER(MbsEquilOptions_c), 
                                              ctypes.c_void_p, ctypes.c_int]
libmodules.mbs_equil_add_variable.restypes = None


libmodules.mbs_new_modal.argtypes =[ctypes.POINTER(MbsData_c)]
libmodules.mbs_new_modal.restype  = ctypes.POINTER(MbsModal_c)


libmodules.mbs_new_invdyn.argtypes =[ctypes.POINTER(MbsData_c)]
libmodules.mbs_new_invdyn.restype  = ctypes.POINTER(MbsInvdyn_c)

libmodules.mbs_run_invdyn.argtypes =[ctypes.POINTER(MbsInvdyn_c), ctypes.POINTER(MbsData_c)]
libmodules.mbs_run_invdyn.restype  = None

libmodules.mbs_invdyn_init.argtypes =[ctypes.POINTER(MbsInvdyn_c), ctypes.POINTER(MbsData_c)]
libmodules.mbs_invdyn_init.restype  = None

libmodules.mbs_invdyn_loop.argtypes =[ctypes.POINTER(MbsInvdyn_c), ctypes.POINTER(MbsData_c)]
libmodules.mbs_invdyn_loop.restype  = None

libmodules.mbs_invdyn_finish.argtypes =[ctypes.POINTER(MbsInvdyn_c), ctypes.POINTER(MbsData_c)]
libmodules.mbs_invdyn_finish.restype  = None

libmodules.mbs_delete_invdyn.argtypes =[ctypes.POINTER(MbsInvdyn_c), ctypes.POINTER(MbsData_c)]
libmodules.mbs_delete_invdyn.restype  = None


libmodules.mbs_new_solvekin.argtypes =[ctypes.POINTER(MbsData_c)]
libmodules.mbs_new_solvekin.restype  = ctypes.POINTER(MbsSolvekin_c)

libmodules.mbs_delete_solvekin.argtypes =[ctypes.POINTER(MbsSolvekin_c), ctypes.POINTER(MbsData_c)]
libmodules.mbs_delete_solvekin.restype  = None

libmodules.mbs_run_solvekin.argtypes =[ctypes.POINTER(MbsSolvekin_c), ctypes.POINTER(MbsData_c)]
libmodules.mbs_run_solvekin.restype  = None

libmodules.mbs_solvekin_init.argtypes =[ctypes.POINTER(MbsSolvekin_c), ctypes.POINTER(MbsData_c)]
libmodules.mbs_solvekin_init.restype  = None

libmodules.mbs_solvekin_loop.argtypes =[ctypes.POINTER(MbsSolvekin_c), ctypes.POINTER(MbsData_c)]
libmodules.mbs_solvekin_loop.restype  = None

libmodules.mbs_solvekin_finish.argtypes =[ctypes.POINTER(MbsSolvekin_c), ctypes.POINTER(MbsData_c)]
libmodules.mbs_solvekin_finish.restype  = None



libmodules.mbs_new_modal.argtypes =[ctypes.POINTER(MbsData_c)]
libmodules.mbs_new_modal.restype  = ctypes.POINTER(MbsModal_c)

libmodules.mbs_delete_modal.argtypes =[ctypes.POINTER(MbsModal_c), ctypes.POINTER(MbsData_c)]
libmodules.mbs_delete_modal.restype  = None

libmodules.mbs_run_modal.argtypes =[ctypes.POINTER(MbsModal_c), ctypes.POINTER(MbsData_c)]
libmodules.mbs_run_modal.restype  = None

libmodules.mbs_modal_init.argtypes =[ctypes.POINTER(MbsModal_c), ctypes.POINTER(MbsData_c)]
libmodules.mbs_modal_init.restype  = None

libmodules.mbs_modal_loop.argtypes =[ctypes.POINTER(MbsModal_c), ctypes.POINTER(MbsData_c)]
libmodules.mbs_modal_loop.restype  = None

libmodules.mbs_modal_finish.argtypes =[ctypes.POINTER(MbsModal_c), ctypes.POINTER(MbsData_c)]
libmodules.mbs_modal_finish.restype  = None