#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
-------------------------------
(c) Universite catholique de Louvain, 2019

Creation : 2019 by LB
Last update : 2019

version MBsysC v1.11.2

-------------------------------
Portable Python interface to MBsysC using Ctypes.


Declaration of MBsysC structure related to mbs_modal_struct.h header file in MBsysC.

"""

import ctypes

from .forward_decl import MbsInfos_c
from .forward_decl import MbsInfoBase_c
from .forward_decl import MbsInfoBodytree_c
from .forward_decl import MbsInfoBody_c
from .forward_decl import MbsInfoJoint_c
from .forward_decl import MbsInfoRefPoint_c
from .forward_decl import MbsInfoCuts_c
from .forward_decl import MbsInfoLinks_c
from .forward_decl import MbsInfoLink_c
from .forward_decl import MbsInfoUserModels_c
from .forward_decl import MbsInfoUserModel_c
from .forward_decl import MbsInfoUserIOs_c
from .forward_decl import MbsInfoPoint_c
from .forward_decl import MbsInfoSensor_c
from .forward_decl import MbsInfoExtforce_c
from .forward_decl import MbsInfoParameter_c
from .forward_decl import MbsInfoOpti_c


#==============================================================================
# MbsInfos_c
#==============================================================================

MbsInfos_c._fields_ = [
    ("base", ctypes.POINTER(MbsInfoBase_c)),
    ("bodytree", ctypes.POINTER(MbsInfoBodytree_c)),
    ("cuts", ctypes.POINTER(MbsInfoCuts_c)),
    ("links", ctypes.POINTER(MbsInfoLinks_c)),
    ("user_models", ctypes.POINTER(MbsInfoUserModels_c)),
    ("user_IOs", ctypes.POINTER(MbsInfoUserIOs_c)),
    ("mbsfile", ctypes.c_char_p),
    ("mbsname", ctypes.c_char_p),
    ("padversion", ctypes.c_char_p),
    ("n_point", ctypes.c_int),
    ("point_list", ctypes.POINTER(ctypes.POINTER(MbsInfoPoint_c))),
    ("n_sensor", ctypes.c_int),
    ("sensor_point_list", ctypes.POINTER(ctypes.c_int)),
    ("n_extforce", ctypes.c_int),
    ("extforce_point_list", ctypes.POINTER(ctypes.c_int)),
    ("n_state", ctypes.c_int),
    ("n_state_value", ctypes.c_int),
    ("state_list", ctypes.POINTER(ctypes.POINTER(MbsInfoParameter_c))),
    ("opti", ctypes.POINTER(MbsInfoOpti_c)),
    ]

#==============================================================================
# MbsInfoBodyTree_c
#==============================================================================

MbsInfoBodytree_c._fields_ = [
    ("n_body", ctypes.c_int),
    ("body_list", ctypes.POINTER(ctypes.POINTER(MbsInfoBody_c))),
    ("n_joint", ctypes.c_int),
    ("joint_list", ctypes.POINTER(ctypes.POINTER(MbsInfoJoint_c)))
    #remaining fields not implemented
    ]

#==============================================================================
# MbsInfoBody_c
#==============================================================================

MbsInfoBody_c._fields_ = [
    ("name", ctypes.c_char_p),
    ("body_list", ctypes.POINTER(ctypes.POINTER(MbsInfoRefPoint_c))),
    ("n_joint", ctypes.c_int),
    ("joint_list", ctypes.POINTER(ctypes.POINTER(MbsInfoJoint_c))),
    ("mass", ctypes.c_double),
    ("com", ctypes.POINTER(ctypes.c_double)),
    ("inertia", ctypes.POINTER(ctypes.c_double)),
    ("n_point", ctypes.c_int),
    ("point_list", ctypes.POINTER(ctypes.POINTER(MbsInfoPoint_c)))
    ]

#==============================================================================
# MbsInfoJoint_c
#==============================================================================

MbsInfoJoint_c._fields_ = [
    ("name", ctypes.c_char_p)
    #remaining fields not implemented
    ]

#==============================================================================
# MbsInfoLinks_c
#==============================================================================

MbsInfoLinks_c._fields_ = [
    ("n_link", ctypes.c_int),
    ("link_list", ctypes.POINTER(ctypes.POINTER(MbsInfoLink_c))),
    ("n_link3D", ctypes.c_int),
    ("link3D_list", ctypes.POINTER(ctypes.POINTER(MbsInfoLink_c))),]


#==============================================================================
# MbsInfoLink_c
#==============================================================================

MbsInfoLink_c._fields_ = [
    ("name", ctypes.c_char_p),
    #remaining fields not implemented
    ]

#==============================================================================
# MbsInfoUserModels_c
#==============================================================================

MbsInfoUserModels_c._fields_ = [
    ("n_user_model", ctypes.c_int),
    ("user_model_list", ctypes.POINTER(ctypes.POINTER(MbsInfoUserModel_c)))
    ]

#==============================================================================
# MbsInfoUserModel_c
#==============================================================================

MbsInfoUserModel_c._fields_ = [
    ("name", ctypes.c_char_p),
    ("n_parameter", ctypes.c_int),
    ("parameter_list", ctypes.POINTER(ctypes.POINTER(MbsInfoParameter_c)))
    ]

#==============================================================================
# MbsInfoPoint_c
#==============================================================================

MbsInfoPoint_c._fields_ = [
    ("name", ctypes.c_char_p),
    ("body_name", ctypes.c_char_p),
    ("pt", ctypes.POINTER(ctypes.c_double)),
    ("sensor", ctypes.POINTER(MbsInfoSensor_c)),
    ("extforce", ctypes.POINTER(MbsInfoExtforce_c)),
    #remaining fields not implemented
    ]

#==============================================================================
# MbsInfoSensor_c
#==============================================================================

MbsInfoSensor_c._fields_ = [
    ("name", ctypes.c_char_p)
    #remaining fields not implemented
    ]

#==============================================================================
# MbsInfoExtforce_c
#==============================================================================

MbsInfoExtforce_c._fields_ = [
    ("name", ctypes.c_char_p)
    #remaining fields not implemented
    ]

#==============================================================================
# MbsInfoParameter_c
#==============================================================================

MbsInfoParameter_c._fields_ = [
    ("name", ctypes.c_char_p),
    ("type", ctypes.c_int),
    ("n_value", ctypes.c_int),
    ("value_list", ctypes.POINTER(ctypes.c_double)),
    ("val_ptr", ctypes.POINTER(ctypes.c_double))
    #remaining fields not implemented
    ]